@@ -13,6 +13,7 @@ import charmhelpers.contrib.openstack.utils as os_utils
 import charmhelpers.contrib.openstack.ha as os_ha
 import charmhelpers.contrib.openstack.ha.utils as os_ha_utils
 import charmhelpers.contrib.openstack.cert_utils as cert_utils
+import charmhelpers.contrib.openstack.ip as ch_os_ip
 import charmhelpers.core.hookenv as hookenv
 import charmhelpers.core.host as ch_host
 import charmhelpers.fetch as fetch
@@ -280,10 +281,14 @@ class OpenStackCharm(BaseOpenStackCharm,
         os_utils.manage_payload_services('stop', self.full_service_list)
         os_utils.manage_payload_services('start', self.full_service_list)
 
-    def get_certificate_requests(self):
-        """Return a dict of certificate requests"""
+    def get_certificate_requests(self, bindings=None):
+        """Return a dict of certificate requests
+
+        :param bindings: List of binding string names for cert requests
+        :type bindings: List[str]
+        """
         return cert_utils.get_certificate_request(
-            json_encode=False).get('cert_requests', {})
+            json_encode=False, bindings=bindings).get('cert_requests', {})
 
     @property
     def rabbit_client_cert_dir(self):
@@ -350,7 +355,8 @@ class OpenStackCharm(BaseOpenStackCharm,
         :rtype: List[str]
         """
         addresses = [
-            os_utils.get_host_ip(hookenv.unit_get('private-address'))]
+            os_utils.get_host_ip(ch_os_ip.local_address(
+                unit_get_fallback='private-address'))]
         for addr_type in os_ip.ADDRESS_MAP.keys():
             laddr = os_ip.resolve_address(endpoint_type=addr_type)
             if laddr:
