@@ -45,6 +45,10 @@ _singleton = None
 # This is to enable the defining code to define which release is used.
 _release_selector_function = None
 
+# `_get_charm_instance_function` holds a function that takes optionally takes a
+# release and returns the corresponding charm class.
+_get_charm_instance_function = None
+
 # `_package_type_selector_function` holds a function that optionally takes a
 # package type and commutes it to another package type or just returns a
 # package type. This is to enable the defining code to define which
@@ -100,7 +104,8 @@ class provide_charm_instance(object):
         return False
 
 
-def get_charm_instance(release=None, package_type='deb', *args, **kwargs):
+def default_get_charm_instance(release=None, package_type='deb', *args,
+                               **kwargs):
     """Get an instance of the charm based on the release (or use the
     default if release is None).
 
@@ -147,6 +152,24 @@ def get_charm_instance(release=None, package_type='deb', *args, **kwargs):
     return cls(release=release, *args, **kwargs)
 
 
+def get_charm_instance(release=None, package_type='deb', *args, **kwargs):
+    """Get an instance of the charm based on the release (or use the
+    default if release is None).
+
+    Use a bespoke method if one is registered otherwise uses the default
+    default_get_charm_instance.
+
+    :param release: lc string representing release wanted.
+    :param package_type: string representing the package type required
+    :returns: BaseOpenStackCharm() derived class according to cls.releases
+    """
+    return (_get_charm_instance_function or default_get_charm_instance)(
+        release=release,
+        package_type=package_type,
+        *args,
+        **kwargs)
+
+
 def register_os_release_selector(f):
     """Register a function that determines what the release is for the
     invocation run.  This allows the charm to define HOW the release is
@@ -171,6 +194,31 @@ def register_os_release_selector(f):
     return f
 
 
+def register_get_charm_instance(f):
+    """Register a function that supplies a charm class for a given
+    release.
+
+    Usage:
+
+        @register_get_charm_instance
+        def my_get_charm_instance(release=None, *args, **kwargs):
+            if release == X:
+                cls = CharmClassX
+            return cls(release=release, *args, **kwargs)
+
+    The function should return a string which is an OS release.
+    """
+    global _get_charm_instance_function
+    if _get_charm_instance_function is None:
+        # we can only do this once in a system invocation.
+        _get_charm_instance_function = f
+    else:
+        raise RuntimeError(
+            "Only a single get_charm_instance is supported."
+            " Called with {}".format(f.__name__))
+    return f
+
+
 def register_package_type_selector(f):
     """Register a function that determines what the package type is for the
     invocation run.  This allows the charm to define HOW the package type is
@@ -488,6 +536,48 @@ class BaseOpenStackCharm(object, metaclass=BaseOpenStackCharmMeta):
         # NOTE(jamespage): fallback to codename assumption
         return version_or_codename
 
+    @staticmethod
+    def get_package_version(package, apt_cache_sufficient=False):
+        """Derive OpenStack release codename from a package.
+
+        :param package: Package name to lookup (ie. in apt cache)
+        :type package: str
+        :param apt_cache_sufficient: When False (the default) version from an
+            installed package will be used, when True version from the systems
+            APT cache will be used.  This is useful for subordinate charms who
+            need working release selection prior to package installation and
+            has no way of using fall back to version of a package the principle
+            charm has installed nor package source configuration option.
+        :type apt_cache_sufficient: bool
+        :returns: OpenStack version name corresponding to package
+        :rtype: Optional[str]
+        :raises: AttributeError, ValueError
+        """
+        cache = fetch.apt_cache()
+
+        try:
+            pkg = cache[package]
+        except KeyError:
+            # the package is unknown to the current apt cache.
+            e = ValueError(
+                'Could not determine version of package with no installation '
+                'candidate: {}'.format(package))
+            raise e
+
+        if apt_cache_sufficient:
+            vers = fetch.apt_pkg.upstream_version(pkg.version)
+        else:
+            vers = fetch.apt_pkg.upstream_version(pkg.current_ver.ver_str)
+
+        # x.y match only for 20XX.X
+        # and ignore patch level for other packages
+        match = re.match(r'^(\d+)\.(\d+)', vers)
+
+        if match:
+            vers = match.group(0)
+
+        return vers
+
     @staticmethod
     def get_os_codename_package(package, codenames, fatal=True,
                                 apt_cache_sufficient=False):
@@ -521,37 +611,18 @@ class BaseOpenStackCharm(object, metaclass=BaseOpenStackCharmMeta):
         :rtype: Optional[str]
         :raises: AttributeError, ValueError
         """
-        cache = fetch.apt_cache()
-
         try:
-            pkg = cache[package]
-        except KeyError:
-            if not fatal:
+            vers = BaseOpenStackCharm.get_package_version(
+                package,
+                apt_cache_sufficient=apt_cache_sufficient)
+            # Generate a major version number for newer semantic
+            # versions of openstack projects
+            major_vers = vers.split('.')[0]
+        except Exception:
+            if fatal:
+                raise
+            else:
                 return None
-            # the package is unknown to the current apt cache.
-            e = ValueError(
-                'Could not determine version of package with no installation '
-                'candidate: {}'.format(package))
-            raise e
-
-        if apt_cache_sufficient:
-            vers = fetch.apt_pkg.upstream_version(pkg.version)
-        else:
-            if not pkg.current_ver:
-                if not fatal:
-                    return None
-            vers = fetch.apt_pkg.upstream_version(pkg.current_ver.ver_str)
-
-        # x.y match only for 20XX.X
-        # and ignore patch level for other packages
-        match = re.match(r'^(\d+)\.(\d+)', vers)
-
-        if match:
-            vers = match.group(0)
-
-        # Generate a major version number for newer semantic
-        # versions of openstack projects
-        major_vers = vers.split('.')[0]
         if (package in codenames and
                 major_vers in codenames[package]):
             return codenames[package][major_vers]
